unit libsndfilestrings;

{$mode objfpc}{$H+}

{*******************************************************************************
*             unit libsndfilestrings  for LibSndFile.dll (Windows)             *
*                                                                              *
*             Addon functions to display STRINGS related to CONST values       *
*                                                                              *
*                                                                              *
*                      created 2011-01-26  (C) breakoutbox                     *
*                      latest changes 2021-09-23  (C) breakoutbox              *
*                                                                              *
*            released under GNU Lesser General Public License (LGPL)           *
*******************************************************************************}

interface

uses
  Classes, SysUtils, ctypes, libsndfile;


function sf_formatmajorstr( sf_format : cint) : String;
function sf_formatsubtypestr( sf_format : cint) : String;


//* Helper functions for sf_command Interface :
//* ---------------------------------------------------------------
//* The Command Interface allows the caller to retrieve information
//* from or change aspects of the library behaviour. Examples include
//* retrieving a string containing the library version or changing
//* the scaling applied to floating point sample data during read and write.
//* Most of these operations are performed on a per-file basis.

function sf_LibSndVersionStr: String;  // ShortString?
function sf_LibSndLogStr( SFhandle: TSNDFILE_HANDLE): String;  // ShortString?
function sf_FileGetMaxSample( SFhandle: TSNDFILE_HANDLE): Double;
function sf_FileGetNormalisedMaxSample( SFhandle: TSNDFILE_HANDLE): Double;
function sf_FileGetMaxAllChannels( SFhandle: TSNDFILE_HANDLE;
                                   number_of_channels:ctypes.cint): Double;

procedure sf_FileGetBroadcastChunk( SFhandle: TSNDFILE_HANDLE; var Result: TStringList);


implementation

uses
  Dialogs;


function sf_formatmajorstr( sf_format : cint) : String;
begin
  //* check on Major formats :  *//
  case (sf_format and $FFFF0000) of
    SF_FORMAT_WAV   : Result:= 'Microsoft WAV format';
    SF_FORMAT_AIFF  : Result:= 'Apple/SGI AIFF format';
    SF_FORMAT_AU    : Result:= 'Sun/NeXT AU format';
    SF_FORMAT_RAW   : Result:= 'RAW PCM data';
    SF_FORMAT_PAF   : Result:= 'Ensoniq PARIS file format';
    SF_FORMAT_SVX   : Result:= 'Amiga IFF / SVX8 / SV16 format';
    SF_FORMAT_NIST  : Result:= 'Sphere NIST format';
    SF_FORMAT_VOC   : Result:= 'VOC file';
    SF_FORMAT_IRCAM : Result:= 'Berkeley/IRCAM/CARL';
    SF_FORMAT_W64   : Result:= 'Sonic Foundry''s 64 bit RIFF/WAV';
    SF_FORMAT_MAT4  : Result:= 'Matlab (tm) V4.2 / GNU Octave 2.0';
    SF_FORMAT_MAT5  : Result:= 'Matlab (tm) V5.0 / GNU Octave 2.1';
    SF_FORMAT_PVF   : Result:= 'Portable Voice Format';
    SF_FORMAT_XI    : Result:= 'Fasttracker 2 Extended Instrument';
    SF_FORMAT_HTK   : Result:= 'HMM Tool Kit format';
    SF_FORMAT_SDS   : Result:= 'Midi Sample Dump Standard';
    SF_FORMAT_AVR   : Result:= 'Audio Visual Research';
    SF_FORMAT_WAVEX : Result:= 'MS WAVE with WAVEFORMATEX';
    SF_FORMAT_SD2   : Result:= 'Sound Designer 2';
    SF_FORMAT_FLAC  : Result:= 'FLAC lossless file format';
    SF_FORMAT_CAF   : Result:= 'Core Audio File format';
    else Result:= 'Undefined format';
  end;
end;


function sf_formatsubtypestr( sf_format : cint) : string;
begin
  //* check on Subtype formats :  *//
  case (sf_format and $0000FFFF) of
    SF_FORMAT_PCM_S8    : Result:= 'Signed 8 bit data';
    SF_FORMAT_PCM_16    : Result:= 'Signed 16 bit data';
    SF_FORMAT_PCM_24    : Result:= 'Signed 24 bit data';
    SF_FORMAT_PCM_32    : Result:= 'Signed 32 bit data';

    SF_FORMAT_PCM_U8    : Result:= 'Unsigned 8 bit data (WAV and RAW only)';

    SF_FORMAT_FLOAT     : Result:= '32 bit float data';
    SF_FORMAT_DOUBLE    : Result:= '64 bit float data';

    SF_FORMAT_ULAW      : Result:= 'U-Law encoded';
    SF_FORMAT_ALAW      : Result:= 'A-Law encoded';
    SF_FORMAT_IMA_ADPCM : Result:= 'IMA ADPCM';
    SF_FORMAT_MS_ADPCM  : Result:= 'Microsoft ADPCM';

    SF_FORMAT_GSM610    : Result:= 'GSM 6.10 encoding';
    SF_FORMAT_VOX_ADPCM : Result:= 'OKI / Dialogix ADPCM';

    SF_FORMAT_G721_32   : Result:= '32kbs G721 ADPCM encoding';
    SF_FORMAT_G723_24   : Result:= '24kbs G723 ADPCM encoding';
    SF_FORMAT_G723_40   : Result:= '40kbs G723 ADPCM encoding';

    SF_FORMAT_DWVW_12   : Result:= '12 bit Delta Width Variable Word encoding';
    SF_FORMAT_DWVW_16   : Result:= '16 bit Delta Width Variable Word encoding';
    SF_FORMAT_DWVW_24   : Result:= '24 bit Delta Width Variable Word encoding';
    SF_FORMAT_DWVW_N    : Result:= 'N bit Delta Width Variable Word encoding';

    SF_FORMAT_DPCM_8    : Result:= '8 bit differential PCM (XI only)';
    SF_FORMAT_DPCM_16   : Result:= '16 bit differential PCM (XI only)';
    else Result:= 'Undefined subformat';
  end;
end;


//* Retrieve the version of the library as a string
function sf_LibSndVersionStr: String;
var buffer: Array [0..127] of Char;
begin
  //* This call will return the length of the retrieved version string *//
  buffer[0]:= #0;  //make compiler happy ..
  if sf_command( NIL, SFC_GET_LIB_VERSION, buffer, length( buffer)) <> 0
    then Result:= buffer
    else Result:= '';
end;


//* Retrieve the log buffer generated when opening a file as a string.
//* This log buffer can often contain a good reason for why libsndfile
//* failed to open a particular file.
function sf_LibSndLogStr( SFhandle: TSNDFILE_HANDLE): String;
var buffer: array [0..1023] of Char;
  res : Cardinal;
begin
  buffer[0]:= #0;  //init first element, it's a PCHAR array !

  //* This call will return the length of the retrieved string *//
  res:= sf_command( SFhandle, SFC_GET_LOG_INFO, buffer, length( buffer));
  if res = 0
    then Result:= buffer
    else Result:= 'Error:  res = ' +IntToStr( res); //for debug ..
end;


//* Retrieve the measured maximum signal value. This involves
//* reading through the whole file which can be slow (!) on large files.
function sf_FileGetMaxSample( SFhandle: TSNDFILE_HANDLE): double;
var d : Double;
begin
  d:= 0;
  //This call will return Zero on success, non-zero otherwise
  if sf_command( SFhandle, SFC_CALC_SIGNAL_MAX, d, sizeof( d)) = 0
    then Result:= d
    else Result:= 1; // = error value
end;


//* Retrieve the measured normalised maximum signal value. This involves
//* reading through the whole file which can be slow on large files.
function sf_FileGetNormalisedMaxSample( SFhandle: TSNDFILE_HANDLE): double;
var d : Double;
begin
  d:= 0;
  //This call will return Zero on success, non-zero otherwise
  if sf_command( SFhandle, SFC_CALC_NORM_SIGNAL_MAX, d, sizeof( d)) = 0
    then Result:= d
    else Result:= 1; // = error value
end;


//* Calculate the peak value (ie a single number) for EACH (!) channel.
//* This involves reading through the whole file which can be slow on large files.
function sf_FileGetMaxAllChannels( SFhandle: TSNDFILE_HANDLE;
                                   number_of_channels:ctypes.cint): double;
var
  d : array of double;
begin
  d:= NIL; // init for compiler's wellness ;-)
  SetLength( d, number_of_channels);

  //This call will return Zero on success, non-zero otherwise
  if sf_command( SFhandle, SFC_CALC_MAX_ALL_CHANNELS, d[0], SizeOf( double)*2) = 0
    then Result:= d[0]
    //*** FIXME:  d[0] is wrong - we need all values of the array here !
    else Result:= 1; // = error value

  SetLength( d, 0);
end;


//* Retrieve the Broadcast Extension Chunk from WAV (and related) files.
procedure sf_FileGetBroadcastChunk( SFhandle: TSNDFILE_HANDLE; var Result: TStringList);
var
  BCI : TSF_BROADCAST_INFO;
begin
  //clean buffer:
  BCI.coding_history_size:= 0;
  FillChar( BCI.coding_history[0], length( BCI.coding_history), 0);

  if sf_command( SFhandle, SFC_GET_BROADCAST_INFO,
                 BCI, sizeof( BCI)) = SF_TRUE
    then
      begin
        Result.Append( 'Description   : ' +BCI.description);
        Result.Append( 'Originator    : ' +BCI.originator);
        Result.Append( 'Orig. ref     : ' +BCI.originator_reference);
        Result.Append( 'Orig. date    : ' +BCI.origination_date);
        Result.Append( 'Orig. time    : ' +BCI.origination_time);
        Result.Append( 'Orig. timeH   : ' +IntToStr( BCI.time_reference_high));
        Result.Append( 'Orig. timeL   : ' +IntToStr( BCI.time_reference_low));
        Result.Append( 'BWF version   : ' +IntToStr( BCI.version));
        //FixMe:  lots of things still missing here !
        Result.Append( 'codinghistory : ' +BCI.coding_history);
      end
    else Result.Append( 'Sorry, but this File has no BWF Chunk.');
end;


end.

